searchNodes=[{"doc":"Provides functions to create composable and bidirectional serializers. Introduction The Codex module provides a number of predefined codecs and combinators that you can use to build new codes. iex&gt; first_codec = sequence ( [ byte ( ) , byte ( ) , byte ( ) ] ) ...&gt; &lt;&lt; 0x10 , 0xFF , 0xAB &gt;&gt; |&gt; decode ( first_codec ) { :ok , [ 16 , 255 , 171 ] , &lt;&lt; &gt;&gt; } ...&gt; [ 16 , 255 , 171 ] |&gt; encode ( first_codec ) { :ok , &lt;&lt; 0x10 , 0xFF , 0xAB &gt;&gt; }","ref":"Codex.html","title":"Codex","type":"module"},{"doc":"Appends a list codec with a codec to form a new list. Examples iex&gt; codec = list_of ( 3 , byte ( ) ) |&gt; append ( bits ( 4 ) ) ...&gt; [ 1 , 2 , 3 , 4 ] |&gt; encode ( codec ) { :ok , &lt;&lt; 1 , 2 , 3 , 4 :: 4 &gt;&gt; } ...&gt; &lt;&lt; 1 , 2 , 3 , 4 :: 4 &gt;&gt; |&gt; decode ( codec ) { :ok , [ 1 , 2 , 3 , 4 ] , &lt;&lt; &gt;&gt; }","ref":"Codex.html#append/2","title":"Codex.append/2","type":"function"},{"doc":"","ref":"Codex.html#bit/0","title":"Codex.bit/0","type":"function"},{"doc":"","ref":"Codex.html#bits/1","title":"Codex.bits/1","type":"function"},{"doc":"","ref":"Codex.html#bits_remaining/0","title":"Codex.bits_remaining/0","type":"function"},{"doc":"","ref":"Codex.html#bool_bit/0","title":"Codex.bool_bit/0","type":"function"},{"doc":"","ref":"Codex.html#byte/0","title":"Codex.byte/0","type":"function"},{"doc":"","ref":"Codex.html#bytes/1","title":"Codex.bytes/1","type":"function"},{"doc":"Tries to encode/decode each codec in succession, using the first one that succeeds. Examples iex&gt; &lt;&lt; 1 &gt;&gt; |&gt; decode ( choice ( [ byte ( ) , bits ( 4 ) ] ) ) { :ok , 1 , &lt;&lt; &gt;&gt; } iex&gt; &lt;&lt; 1 :: 4 &gt;&gt; |&gt; decode ( choice ( [ byte ( ) , bits ( 4 ) ] ) ) { :ok , 1 , &lt;&lt; &gt;&gt; } iex&gt; &lt;&lt; 1 :: 2 &gt;&gt; |&gt; decode ( choice ( [ byte ( ) , bits ( 4 ) ] ) ) { :error , &quot;None of the choices worked&quot; , &lt;&lt; 1 :: 2 &gt;&gt; }","ref":"Codex.html#choice/1","title":"Codex.choice/1","type":"function"},{"doc":"Combines two codecs into a codec that produces a 2-element tuple of each value. Examples iex&gt; { 198 , 2 } |&gt; encode ( combine ( byte ( ) , byte ( ) ) ) { :ok , &lt;&lt; 198 , 2 &gt;&gt; } iex&gt; &lt;&lt; 198 , 2 &gt;&gt; |&gt; decode ( combine ( byte ( ) , byte ( ) ) ) { :ok , { 198 , 2 } , &lt;&lt; &gt;&gt; }","ref":"Codex.html#combine/2","title":"Codex.combine/2","type":"function"},{"doc":"Combines a codec with a list codec to form a new list. Examples iex&gt; non_empty_list = byte ( ) |&gt; cons ( list ( byte ( ) ) ) ...&gt; [ 1 ] |&gt; encode ( non_empty_list ) { :ok , &lt;&lt; 1 &gt;&gt; } ...&gt; [ ] |&gt; encode ( non_empty_list ) { :error , &quot;Failed to encode []&quot; } ...&gt; &lt;&lt; 1 &gt;&gt; |&gt; decode ( non_empty_list ) { :ok , [ 1 ] , &lt;&lt; &gt;&gt; } ...&gt; &lt;&lt; &gt;&gt; |&gt; decode ( non_empty_list ) { :error , &quot;Could not decode 8 bits from \\&quot; \\&quot; &quot; , &lt;&lt; &gt;&gt; }","ref":"Codex.html#cons/2","title":"Codex.cons/2","type":"function"},{"doc":"A codec that always encodes to and decodes from the same value. It fails if it doesn't see the expected value that it always encodes/decodes. Examples iex&gt; &lt;&lt; 200 &gt;&gt; |&gt; decode ( constant ( 10 , &lt;&lt; 200 &gt;&gt; ) ) { :ok , 10 , &lt;&lt; &gt;&gt; } iex&gt; &lt;&lt; 234 &gt;&gt; |&gt; decode ( constant ( 10 , &lt;&lt; 200 &gt;&gt; ) ) { :error , &quot;&lt;&lt;234&gt;&gt; did not equal &lt;&lt;200&gt;&gt; in constant&quot; , &lt;&lt; 234 &gt;&gt; } iex&gt; 10 |&gt; encode ( constant ( 10 , &lt;&lt; 200 &gt;&gt; ) ) { :ok , &lt;&lt; 200 &gt;&gt; } iex&gt; 22 |&gt; encode ( constant ( 10 , &lt;&lt; 200 &gt;&gt; ) ) { :error , &quot;22 did not equal 10 in constant&quot; }","ref":"Codex.html#constant/2","title":"Codex.constant/2","type":"function"},{"doc":"Used to convert a codec to another type. Must also be used with caution to make sure your conversion is idempotent. Examples defmodule Foo do defstruct a : nil , b : nil end iex&gt; tuple_codec = combine ( byte ( ) , byte ( ) ) ...&gt; struct_codec = tuple_codec |&gt; convert ( fn { a , b } -&gt; % Foo { a : a , b : b } end , fn % Foo { a : a , b : b } -&gt; { a , b } end ) ...&gt; &lt;&lt; 12 , 34 &gt;&gt; |&gt; decode ( struct_codec ) { :ok , % Foo { a : 12 , b : 34 } , &lt;&lt; &gt;&gt; } ...&gt; % Foo { a : 12 , b : 34 } |&gt; encode ( struct_codec ) { :ok , &lt;&lt; 12 , 34 &gt;&gt; }","ref":"Codex.html#convert/3","title":"Codex.convert/3","type":"function"},{"doc":"Creates a new codec. It is recommended to use the other, well tested, functions in the module to build a codec instead of creating your own unless you absolutely have to. When creating your own codec it MUST be idempotent. Examples iex&gt; Codex . create ( fn _ -&gt; { :ok , &lt;&lt; &gt;&gt; } end , fn ...&gt; &lt;&lt; &gt;&gt; -&gt; { :ok , false , &lt;&lt; &gt;&gt; } ...&gt; bits -&gt; { :ok , true , bits } ...&gt; end ) # Codex &lt; ... &gt;","ref":"Codex.html#create/2","title":"Codex.create/2","type":"function"},{"doc":"Decodes a value from its binary form using the supplied codec. The third element of the tuple is any remaining unparsed bits. Examples iex&gt; &lt;&lt; 198 &gt;&gt; |&gt; decode ( byte ( ) ) { :ok , 198 , &lt;&lt; &gt;&gt; }","ref":"Codex.html#decode/2","title":"Codex.decode/2","type":"function"},{"doc":"Only succeeds if there were no bits left to parse. Examples iex&gt; &lt;&lt; 10 &gt;&gt; |&gt; decode ( byte ( ) |&gt; done ( ) ) { :ok , 10 , &lt;&lt; &gt;&gt; } iex&gt; &lt;&lt; 10 , 11 &gt;&gt; |&gt; decode ( byte ( ) |&gt; done ( ) ) { :error , &quot;There was more to parse&quot; , &lt;&lt; 11 &gt;&gt; }","ref":"Codex.html#done/1","title":"Codex.done/1","type":"function"},{"doc":"A codec that always decodes to an empty list, [] . It can never fail decoding. Examples iex&gt; defmodule EmptyExample do ...&gt; def listify ( [ ] ) , do : empty ( ) ...&gt; def listify ( [ codec | rest ] ) , do : codec |&gt; cons ( listify ( rest ) ) ...&gt; end ...&gt; &lt;&lt; 1 &gt;&gt; |&gt; decode ( EmptyExample . listify ( [ ] ) ) { :ok , [ ] , &lt;&lt; 1 &gt;&gt; } ...&gt; &lt;&lt; 22 , 38 &gt;&gt; |&gt; decode ( EmptyExample . listify ( [ byte ( ) , byte ( ) ] ) ) { :ok , [ 22 , 38 ] , &lt;&lt; &gt;&gt; }","ref":"Codex.html#empty/0","title":"Codex.empty/0","type":"function"},{"doc":"Encodes a value to its binary form using the supplied codec. Examples iex&gt; 198 |&gt; encode ( byte ( ) ) { :ok , &lt;&lt; 198 &gt;&gt; }","ref":"Codex.html#encode/2","title":"Codex.encode/2","type":"function"},{"doc":"","ref":"Codex.html#ensure/3","title":"Codex.ensure/3","type":"function"},{"doc":"Creates a codec that always fails with the supplied error message. This is not useful on its own but can be useful when building other codecs. Examples iex&gt; defmodule FailExample do ...&gt; def choose ( [ ] ) , do : fail ( &quot;None of the choices worked&quot; ) ...&gt; def choose ( [ codec | rest ] ) , do : fallback ( codec , choice ( rest ) ) ...&gt; end ...&gt; codec = FailExample . choose ( [ byte ( ) , bits ( 4 ) ] ) ...&gt; &lt;&lt; 1 &gt;&gt; |&gt; decode ( codec ) { :ok , 1 , &lt;&lt; &gt;&gt; } ...&gt; &lt;&lt; 1 :: 2 &gt;&gt; |&gt; decode ( codec ) { :error , &quot;None of the choices worked&quot; , &lt;&lt; 1 :: 2 &gt;&gt; }","ref":"Codex.html#fail/1","title":"Codex.fail/1","type":"function"},{"doc":"Creates a codec that always fails with the supplied error messages. Same as fail/1 but you can supply a different error message for encoding and decoding.","ref":"Codex.html#fail/2","title":"Codex.fail/2","type":"function"},{"doc":"Uses the first codec, falling back to the second if it fails. Examples iex&gt; optional_byte = byte ( ) |&gt; fallback ( nothing ( ) ) ...&gt; &lt;&lt; 8 &gt;&gt; |&gt; decode ( optional_byte ) { :ok , 8 , &lt;&lt; &gt;&gt; } ...&gt; &lt;&lt; 8 :: 4 &gt;&gt; |&gt; decode ( optional_byte ) { :ok , nil , &lt;&lt; 8 :: 4 &gt;&gt; }","ref":"Codex.html#fallback/2","title":"Codex.fallback/2","type":"function"},{"doc":"","ref":"Codex.html#join/2","title":"Codex.join/2","type":"function"},{"doc":"","ref":"Codex.html#length_prefixed/2","title":"Codex.length_prefixed/2","type":"function"},{"doc":"","ref":"Codex.html#list/1","title":"Codex.list/1","type":"function"},{"doc":"","ref":"Codex.html#list_of/2","title":"Codex.list_of/2","type":"function"},{"doc":"Tests whether a codec would succeed. Decodes to true if it would, false otherwise. Similar to recover/1 except it doesn't consume the bits when it succeeds and returns true . Examples iex&gt; &lt;&lt; 38 &gt;&gt; |&gt; decode ( lookahead ( byte ( ) ) ) { :ok , true , &lt;&lt; 38 &gt;&gt; } iex&gt; &lt;&lt; 1 :: 1 &gt;&gt; |&gt; decode ( lookahead ( byte ( ) ) ) { :ok , false , &lt;&lt; 1 :: 1 &gt;&gt; }","ref":"Codex.html#lookahead/1","title":"Codex.lookahead/1","type":"function"},{"doc":"","ref":"Codex.html#non_empty_list/1","title":"Codex.non_empty_list/1","type":"function"},{"doc":"Inverts a boolean codec. Examples iex&gt; &lt;&lt; 38 &gt;&gt; |&gt; decode ( not_ ( recover ( byte ( ) ) ) ) { :ok , false , &lt;&lt; &gt;&gt; } iex&gt; &lt;&lt; 1 :: 1 &gt;&gt; |&gt; decode ( not_ ( recover ( byte ( ) ) ) ) { :ok , true , &lt;&lt; 1 :: 1 &gt;&gt; }","ref":"Codex.html#not_/1","title":"Codex.not_/1","type":"function"},{"doc":"A codec that always decodes to nil . It can never fail decoding. Examples iex&gt; optional_byte = byte ( ) |&gt; fallback ( nothing ( ) ) ...&gt; &lt;&lt; 8 &gt;&gt; |&gt; decode ( optional_byte ) { :ok , 8 , &lt;&lt; &gt;&gt; } ...&gt; &lt;&lt; 8 :: 4 &gt;&gt; |&gt; decode ( optional_byte ) { :ok , nil , &lt;&lt; 8 :: 4 &gt;&gt; }","ref":"Codex.html#nothing/0","title":"Codex.nothing/0","type":"function"},{"doc":"Creates a codec that might not work. If it fails decoding it returns nil instead. Examples iex&gt; &lt;&lt; 11 &gt;&gt; |&gt; decode ( optional ( byte ( ) ) ) { :ok , 11 , &lt;&lt; &gt;&gt; } iex&gt; &lt;&lt; 1 :: 4 &gt;&gt; |&gt; decode ( optional ( byte ( ) ) ) { :ok , nil , &lt;&lt; 1 :: 4 &gt;&gt; }","ref":"Codex.html#optional/1","title":"Codex.optional/1","type":"function"},{"doc":"","ref":"Codex.html#pad/2","title":"Codex.pad/2","type":"function"},{"doc":"Creates a codec that decodes without actually consuming the bits. Examples iex&gt; &lt;&lt; 4 &gt;&gt; |&gt; decode ( peek ( byte ( ) ) ) { :ok , 4 , &lt;&lt; 4 &gt;&gt; } iex&gt; codec = peek ( byte ( ) ) |&gt; combine ( byte ( ) ) ...&gt; &lt;&lt; 4 &gt;&gt; |&gt; decode ( codec ) { :ok , { 4 , 4 } , &lt;&lt; &gt;&gt; } ...&gt; { 4 , 4 } |&gt; encode ( codec ) { :ok , &lt;&lt; 4 &gt;&gt; }","ref":"Codex.html#peek/1","title":"Codex.peek/1","type":"function"},{"doc":"Tests whether a codec would succeed. Decodes to true if it would, false otherwise. Examples iex&gt; &lt;&lt; 38 &gt;&gt; |&gt; decode ( recover ( byte ( ) ) ) { :ok , true , &lt;&lt; &gt;&gt; } iex&gt; &lt;&lt; 1 :: 1 &gt;&gt; |&gt; decode ( recover ( byte ( ) ) ) { :ok , false , &lt;&lt; 1 :: 1 &gt;&gt; }","ref":"Codex.html#recover/1","title":"Codex.recover/1","type":"function"},{"doc":"","ref":"Codex.html#refute/3","title":"Codex.refute/3","type":"function"},{"doc":"Reverses a list codec. Examples iex&gt; codec = list ( byte ( ) ) |&gt; reverse ( ) ...&gt; [ 1 , 2 , 3 ] |&gt; encode ( codec ) { :ok , &lt;&lt; 3 , 2 , 1 &gt;&gt; } ...&gt; &lt;&lt; 3 , 2 , 1 &gt;&gt; |&gt; decode ( codec ) { :ok , [ 1 , 2 , 3 ] , &lt;&lt; &gt;&gt; }","ref":"Codex.html#reverse/1","title":"Codex.reverse/1","type":"function"},{"doc":"Combines a list of codecs into a single codec that produces a list of those values . ## Examples iex&gt; codec = sequence ( [ byte ( ) , byte ( ) , byte ( ) ] ) ...&gt; &lt;&lt; 0x10 , 0xFF , 0xAB &gt;&gt; |&gt; decode ( codec ) { :ok , [ 16 , 255 , 171 ] , &lt;&lt; &gt;&gt; } ...&gt; [ 16 , 255 , 171 ] |&gt; encode ( codec ) { :ok , &lt;&lt; 0x10 , 0xFF , 0xAB &gt;&gt; }","ref":"Codex.html#sequence/1","title":"Codex.sequence/1","type":"function"},{"doc":"","ref":"Codex.html#take_until/2","title":"Codex.take_until/2","type":"function"},{"doc":"","ref":"Codex.html#take_while/2","title":"Codex.take_while/2","type":"function"},{"doc":"Use the result of a codec to create the next codec. Examples iex&gt; length = byte ( ) ...&gt; length_prefixed = length |&gt; then ( &amp; list_of ( &amp;1 , byte ( ) ) , &amp; length / 1 ) ...&gt; &lt;&lt; 4 , 1 , 2 , 3 , 4 &gt;&gt; |&gt; decode ( length_prefixed ) { :ok , [ 1 , 2 , 3 , 4 ] , &lt;&lt; &gt;&gt; } ...&gt; [ 1 , 2 , 3 , 4 ] |&gt; encode ( length_prefixed ) { :ok , &lt;&lt; 4 , 1 , 2 , 3 , 4 &gt;&gt; }","ref":"Codex.html#then/3","title":"Codex.then/3","type":"function"},{"doc":"A codec that always decodes to a certain value. It never consumes bits while decoding so it can't fail. It can fail on encoding if the value to encode doesn't match. Examples iex&gt; &lt;&lt; &gt;&gt; |&gt; decode ( value ( 10 ) ) { :ok , 10 , &lt;&lt; &gt;&gt; } iex&gt; &lt;&lt; 200 &gt;&gt; |&gt; decode ( value ( 10 ) ) { :ok , 10 , &lt;&lt; 200 &gt;&gt; } iex&gt; 10 |&gt; encode ( value ( 10 ) ) { :ok , &lt;&lt; &gt;&gt; } iex&gt; 22 |&gt; encode ( value ( 10 ) ) { :error , &quot;22 did not equal 10 in constant&quot; }","ref":"Codex.html#value/1","title":"Codex.value/1","type":"function"},{"doc":"","ref":"Codex.html#t:boolean_codec/0","title":"Codex.boolean_codec/0","type":"type"},{"doc":"","ref":"Codex.html#t:codec/1","title":"Codex.codec/1","type":"type"},{"doc":"","ref":"Codex.html#t:decode_result/1","title":"Codex.decode_result/1","type":"type"},{"doc":"","ref":"Codex.html#t:decoder/1","title":"Codex.decoder/1","type":"type"},{"doc":"","ref":"Codex.html#t:encode_result/0","title":"Codex.encode_result/0","type":"type"},{"doc":"","ref":"Codex.html#t:encoder/1","title":"Codex.encoder/1","type":"type"},{"doc":"","ref":"Codex.html#t:list_codec/1","title":"Codex.list_codec/1","type":"type"},{"doc":"","ref":"Codex.html#t:remaining_bits/0","title":"Codex.remaining_bits/0","type":"type"},{"doc":"","ref":"Codex.html#t:type/0","title":"Codex.type/0","type":"type"},{"doc":"","ref":"Codex.html#t:type2/0","title":"Codex.type2/0","type":"type"},{"doc":"","ref":"Codex.Base64.html","title":"Codex.Base64","type":"module"},{"doc":"","ref":"Codex.Base64.html#codec/0","title":"Codex.Base64.codec/0","type":"function"},{"doc":"","ref":"Codex.Gve.html","title":"Codex.Gve","type":"module"},{"doc":"","ref":"Codex.Gve.html#codec/0","title":"Codex.Gve.codec/0","type":"function"},{"doc":"","ref":"Codex.Util.html","title":"Codex.Util","type":"module"},{"doc":"","ref":"Codex.Util.html#bits_to_int/1","title":"Codex.Util.bits_to_int/1","type":"function"},{"doc":"","ref":"Codex.Util.html#join/2","title":"Codex.Util.join/2","type":"function"},{"doc":"","ref":"Codex.Util.html#remove_leading_zeros/1","title":"Codex.Util.remove_leading_zeros/1","type":"function"},{"doc":"","ref":"Codex.Util.html#reverse_bits/1","title":"Codex.Util.reverse_bits/1","type":"function"},{"doc":"","ref":"Codex.Util.html#split/2","title":"Codex.Util.split/2","type":"function"},{"doc":"","ref":"Codex.Varint.html","title":"Codex.Varint","type":"module"},{"doc":"","ref":"Codex.Varint.html#codec/0","title":"Codex.Varint.codec/0","type":"function"}]